package de.mpmediasoft.wasm.library

import de.mpmediasoft.wasm.SystemFileSystem
import de.mpmediasoft.wasm.readBytes
import io.github.charlietap.chasm.embedding.*
import io.github.charlietap.chasm.embedding.shapes.*
import io.github.charlietap.chasm.executor.runtime.value.NumberValue
import okio.Path.Companion.toPath

class LibraryWasmChasmImpl : Library {

    companion object {
        private val wasmDir = "/Users/mpaus/Projects/tmp/KotlinWasmBridgeDemo/libs/".toPath()

        // "r_fibonacci.wasm" OK, "c_fibonacci.wasm" OK, "j_fibonacci.wasm" FAILS
        private val fibWasmFile = "r_fibonacci.wasm"
        private val fibFile = wasmDir.resolve(fibWasmFile)
        private val fibStore = store()
        private var fibInstance: Instance
        private val fibName = "fibonacci"

        // "r_count_vowels.wasm" TODO, "c_count_vowels.wasm" TODO
        private val vowFile = wasmDir.resolve("r_count_vowels.wasm") // "r_count_vowels.wasm", "c_count_vowels.wasm"
        private val vowStore = store()
        private var vowInstance: Instance
        private val vowExports: List<Export>
        private val vowName = "countVowels"

        private fun instantiateModule(store: Store, byteArray: ByteArray): Instance {
            return module(byteArray).flatMap { module -> instance(store, module, emptyList()) }.expect("Failed to instantiate module")
        }

        init {
            require(SystemFileSystem.exists(wasmDir)) { "Wasm directory does not exist." }

            fibInstance = instantiateModule(fibStore, fibFile.readBytes())
            if (fibWasmFile == "c_fibonacci.wasm") invoke(fibStore, fibInstance, "_initialize").expect("Failed to calculate fibonacci")

            vowInstance = instantiateModule(vowStore, vowFile.readBytes())
            vowExports = exports(vowInstance)
            vowExports.forEach { e -> println("Export: ${e.name}")  }

            // TODO:
        }
    }

    private fun vowAlloc(len: Int): Int {
        return invoke(vowStore, vowInstance, "alloc", listOf(NumberValue.I32(len))).map {(it.first() as NumberValue.I32).value}.expect("Failed to allocate memory.")
    }

    override fun calculateFibonacci(n: Int): Int {
        return invoke(fibStore, fibInstance, fibName, listOf(NumberValue.I32(n))).map {(it.first() as NumberValue.I32).value}.expect("Failed to calculate fibonacci")
    }

    override fun countVowels(message: String): Int {
        var len = message.encodeToByteArray().size
        var ptr = vowAlloc(len)

        
//        val memory = vowExports.filterIsInstance<Memory>().first()
//
//        val buffer = byteArrayOf()
//        val bytes = io.github.charlietap.chasm.embedding.memory.readBytes(vowStore, memory, buffer, ptr, len)




        TODO("Not yet implemented")
    }

}