package de.mpmediasoft.wasm.library

import de.mpmediasoft.wasm.readBytes
import com.dylibso.chicory.experimental.aot.AotMachine
import com.dylibso.chicory.runtime.ExportFunction
import com.dylibso.chicory.runtime.Instance
import com.dylibso.chicory.runtime.Memory
import com.dylibso.chicory.wasm.Parser
import okio.Path.Companion.toPath

class LibraryWasmChicoryImpl : Library {

    // Not used yet but also see: https://github.com/extism/chicory-sdk

    private companion object {
        val wasmDir = "/Users/mpaus/Projects/tmp/KotlinWasmBridgeDemo/libs/".toPath()

        private fun instantiateModule(wasmBytes: ByteArray): Instance {
            return Instance.builder(Parser.parse(wasmBytes)).withMachineFactory(::AotMachine).build()
        }

        // calculateFibonacci:

        // "r_fibonacci.wasm" OK, "c_fibonacci.wasm" OK, "j_fibonacci.wasm" FAILS
        private val fibWasmFile = "j_fibonacci.wasm"
        private val fibInstance : Instance        = instantiateModule(wasmDir.resolve(fibWasmFile).readBytes())
        private val fibonacci   : ExportFunction  = fibInstance.export("fibonacci")

        // countVowels

        // "r_count_vowels.wasm" OK, "c_count_vowels.wasm" FAILS
        private var cwInstance  : Instance       = instantiateModule(wasmDir.resolve("r_count_vowels.wasm").readBytes())
        private var memory      : Memory         = cwInstance.memory()
        private var alloc       : ExportFunction = cwInstance.export("alloc")
        private var dealloc     : ExportFunction = cwInstance.export("dealloc")
        private var countVowels : ExportFunction = cwInstance.export("count_vowels")

        init {
            if (fibWasmFile == "c_fibonacci.wasm") fibInstance.export("_initialize").apply()
        }
    }

    override fun calculateFibonacci(n: Int): Int {
        return fibonacci.apply(n.toLong())[0].toInt()
    }

    override fun countVowels(message: String): Int {
        val bytes = message.encodeToByteArray()
        val len = bytes.size.toLong()
        val ptr = alloc.apply(len)[0]
        try {
            memory.write(ptr.toInt(), bytes)
            return countVowels.apply(ptr, len)[0].toInt()
        } finally {
            dealloc.apply(ptr, len);
        }
    }

}