package de.mpmediasoft.wasm.library

import org.graalvm.polyglot.Context
import org.graalvm.polyglot.Source
import org.graalvm.polyglot.Value
import java.io.File
import org.graalvm.polyglot.*;
import org.graalvm.polyglot.io.ByteSequence;
import java.nio.file.Files
import java.nio.file.Path


class LibraryWasmGraalImpl : Library {

    private companion object {
        private val wasmDir = "/Users/mpaus/Projects/tmp/KotlinWasmBridgeDemo/libs/"

        // "r_fibonacci.wasm" OK, "c_fibonacci.wasm" OK, "j_fibonacci.wasm" FAILS
        private val fibWasmFile = "j_fibonacci.wasm"
        private val binary: ByteArray = Files.readAllBytes(Path.of(wasmDir, fibWasmFile));
        private val contextBuilder: Context.Builder = Context.newBuilder("wasm")
        private val sourceBuilder: Source.Builder = Source.newBuilder("wasm", ByteSequence.create(binary), "example")
        private val source: Source = sourceBuilder.build()
        private val context: Context = contextBuilder.build()
        private val mainFunction: Value

        init {
            context.eval(source);
            if (fibWasmFile == "c_fibonacci.wasm") context.getBindings("wasm").getMember("example").getMember("_initialize").executeVoid(); // Only for emscripten!
            mainFunction = context.getBindings("wasm").getMember("example").getMember("fibonacci");
        }

        fun dispose() {
            context.close();
        }
    }

    override fun calculateFibonacci(n: Int): Int {
        val res = mainFunction.execute(n)
        return res.asInt()
    }

    override fun countVowels(message: String): Int {
        TODO("Not yet implemented")
    }

}